import { rm, run, resolveBin } from '@genesislcap/build-kit';
import type { BuildContext } from '@genesislcap/build-kit';
import consola from 'consola';

export default async (ctx: BuildContext) => {
  const {
    dirs: { cwd },
    cli: { isDev, isBuild, options },
    config: { clean, copyAssets },
    enable: { apiExtractor, cemAnalyzer },
  } = ctx;

  if (clean) {
    await rm(['dist', 'tsconfig.tsbuildinfo']);
  }

  if (Array.isArray(copyAssets)) {
    const copyfiles = await resolveBin('copyfiles');
    copyAssets.forEach((glob) => {
      run(cwd, `${copyfiles} --verbose --up 1 src/${glob} ./dist/esm`);
    });
  }

  const tsc = await resolveBin('tsc', 'typescript');
  if (isDev) {
    // build a composite project in the working directory in watch mode
    run(cwd, `${tsc} -b ./tsconfig.json --verbose -w`);
    return;
  } else if (isBuild) {
    // build a composite project in the working directory
    run(cwd, `${tsc} -b ./tsconfig.json --verbose`);

    if (apiExtractor) {
      consola.log('\n@microsoft/api-extractor configuration detected');
      const extractor = await resolveBin('api-extractor', '@microsoft/api-extractor');
      run(cwd, `${extractor} run`);

      const documenter = await resolveBin('api-documenter', '@microsoft/api-documenter');
      run(cwd, `${documenter} markdown -i dist -o docs/api`);
    }

    if (cemAnalyzer) {
      consola.log('\n@custom-elements-manifest/analyzer configuration detected');
      const cem = await resolveBin('cem', '@custom-elements-manifest/analyzer');
      run(cwd, `${cem} analyze --config custom-elements-manifest.config.*js`);
    }
  } else {
    throw new Error(`Unrecognized command: ${JSON.stringify(options)}`);
  }
};
